/*
 *   NVRAM WakeUp
 *   Copyright (C) 2001-2003, Sergei Haller.
 *   Copyright (C) 2002 Bernhard "Bero" Rosenkraenzer <bero@arklinux.org>
 *
 *   $Id: nvram-wakeup-mb.c,v 1.62 2003/08/07 15:05:29 bistr-o-math Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#define CVSREV_nvram_wakeup_mb_c \
     "$Id: nvram-wakeup-mb.c,v 1.62 2003/08/07 15:05:29 bistr-o-math Exp $"

#include <stdio.h>
#include <string.h>

#include "nvram-wakeup.h"
#include "nvram-wakeup-mb.h"

/*
 *  If you can't find your motherboard here, try at the MB tracker
 *    http://sourceforge.net/tracker/index.php?group_id=35022&atid=412959
 *  or the latest CVS revision of this file
 *    http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/nvram-wakeup/nvram-wakeup/nvram-wakeup-mb.c?rev=HEAD
 *  or
 *    read README.mb.
 */

typedef void (*info_writer)(struct biosinfo *b);
     
struct mainboard {
     info_writer infowriter;

     char *iw_name;     /* the function name of the infowriter */

     char *vendor;
     char *type;
     char *version;

     char *biosvendor;
     char *biosversion;
     char *biosrelease;
};


/*
 *  look into nvram-wakeup.conf(5) for the documentation of the options
 */
 
/* 
 *  By far the most common way of storing the timing information is having
 *  it in subsequent bytes in the date-hour-min-sec order. #define this
 *  here to save code later on
 */
 
#define TIMEREGS(b, base) b->addr_date = base;  \
                          b->addr_hour = base+1;\
                          b->addr_min  = base+2;\
                          b->addr_sec  = base+3

/* 
 *  Virtually every board keeps the low and high bytes of the checksum in
 *  subsequent bytes - save some more code
 */
 
#define CHECKSUM(b, base) b->addr_chk_h = base;\
                          b->addr_chk_l = base+1

#define   BCD(b) b->nr_mon      = 5;\
                 b->nr_date     = 6;\
                 b->nr_hour     = 6;\
                 b->nr_min      = 7;\
                 b->nr_sec      = 7;\
                 b->nr_rtc_mon  = 5;\
                 b->nr_rtc_date = 6;\
                 b->nr_wdays    = 7;\
                 b->bcd         = ON

#define NOBCD(b) b->nr_mon      = 4;\
                 b->nr_date     = 5;\
                 b->nr_hour     = 5;\
                 b->nr_min      = 6;\
                 b->nr_sec      = 6;\
                 b->nr_rtc_date = 6;\
                 b->nr_rtc_mon  = 5;\
                 b->nr_wdays    = 7

void msi_ms_6351(struct biosinfo *b) 
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x56);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x55;
     b->shift_stat  = 3;
}

void m6vcg_p6bat_8kta3_common(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     b->need_reboot = ON_STAT;
     b->addr_stat   = 0x47;
     b->addr_hour   = 0x41;
     b->addr_min    = 0x40;
     b->addr_sec    = 0x3F;
     b->shift_stat  = 5;
}

void chaintech_6aja4(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x43;
     b->need_reboot = ALWAYS;
}

void ecs_p6bat(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x4F;
}

void shuttle_mvp3(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x4F;
     b->need_reboot = ALWAYS;
}

void bio_m6vcg(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x51;
}

void msi_ms_6368(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x51;
     b->need_reboot = ALWAYS;
}

void epx_8kta3(struct biosinfo *b)
{
     m6vcg_p6bat_8kta3_common(b);
     b->addr_date   = 0x53;
}

void abit_bp6(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     TIMEREGS(b, 0x47);
     b->addr_stat   = 0x46;
}

void aopen_ax4pe_max(struct biosinfo *b)
{
     TIMEREGS(b, 0x47);
     b->addr_stat   = 0x46;
     b->need_reboot = ALWAYS;
}

void ga_6bxc_qdi_common(struct biosinfo *b)
{
     CHECKSUM(b, 0x6F);
     TIMEREGS(b, 0x47);
     b->addr_stat   = 0x46;
}

void gig_ga_6bxc(struct biosinfo *b)
{
     ga_6bxc_qdi_common(b);
     b->need_reboot = ALWAYS;
}

void msi_ms_6160(struct biosinfo *b)
{
     ga_6bxc_qdi_common(b);
     b->need_reboot = ALWAYS;
     b->rtc_time    = ON;
}

void asus_tusl2(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     b->need_reboot = OFF;
     b->rtc_time    = ON;
     b->reset_date  = ON;
     b->addr_stat   = 0x56;
     b->rtc_date    = 0x7F;
     b->shift_stat  = 4;
}

void msi_ms_6120(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 5;
}

void msi_ms_6593(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 3;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void msi_ms_6590(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 6;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void asus_cuv4xe(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     b->need_reboot = OFF; /* ??? */
     b->rtc_time    = ON;
     b->reset_date  = ON;
     b->addr_stat   = 0x56;
     b->rtc_date    = 0x6F;
     b->shift_stat  = 4;
}

void asus_a7s_vm(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->rtc_time    = ON;
     b->addr_stat   = 0x4B;
     b->addr_date   = 0x4C;
     b->shift_stat  = 7;
}

void asus_p4s533(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->rtc_time    = ON;
     b->addr_stat   = 0x4A;
     b->addr_date   = 0x4A;
     b->shift_stat  = 1;
     b->shift_date  = 2;
}

void asus_p2l97s(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     b->need_reboot = OFF;
     b->rtc_time    = ON;
     b->addr_stat   = 0x40;
     b->rtc_date    = 0x7F; /* Needs kernel patch */
     b->shift_stat  = 1;
}

void asus_tx97_xe(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     b->need_reboot = OFF;
     b->rtc_time    = ON;
     b->addr_stat   = 0x41;
     b->rtc_date    = 0x7F; /* Needs kernel patch */
     b->shift_stat  = 2;
}

void pb_bora_pro(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x5A;
     b->shift_date  = 1;
}

void fic_va_503p(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     TIMEREGS(b, 0x49);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x48;
     b->addr_mon    = 0x4D;
     b->shift_stat  = 7;
     b->shift_mon   = 4;
}

void gig_ga_7zx_common(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x57);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x57;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void gig_ga_7zx(struct biosinfo *b)
{
     gig_ga_7zx_common(b);
     b->shift_stat  = 1;
     b->shift_date  = 2;
}

void gig_ga_7zxe(struct biosinfo *b)
{
     gig_ga_7zx_common(b);
     b->shift_date  = 1;
}

void gig_ga_7vtxh(struct biosinfo *b)
{
     gig_ga_7zx_common(b);
     TIMEREGS(b, 0x59);
     b->shift_stat  = 7;
}

void abit_kt7e(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x47;
     b->addr_date   = 0x51;
     b->addr_hour   = 0x41;
     b->addr_min    = 0x40;
     b->shift_stat  = 5;
}

void msi_ms_6570(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x46;
     b->shift_stat  = 3;
     b->addr_date   = 0x48;
     b->addr_hour   = 0x49;
     b->addr_min    = 0x4A;
}

void asus_a7n8x(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x46;
     b->shift_stat  = 3;
     b->addr_hour   = 0x48;
     b->addr_min    = 0x49;
     b->addr_sec    = 0x4A;
}

void msi_ms_6380(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x59);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x58;
     b->shift_stat  = 3;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void msi_ms_6380e(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 5;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void msi_ms_6712(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5A);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x58;
     b->shift_stat  = 7;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void asrock_k7vt2(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x59);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 1;
     b->shift_date  = 2;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void asrock_k7vm2(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x59);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x59;
     b->shift_stat  = 0;
     b->shift_date  = 1;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void asus_sp98_x(struct biosinfo *b)
{
     CHECKSUM(b, 0x6C);
     b->need_reboot = ON_STAT|ON_WDAYS;
     b->rtc_time    = ON;
     b->addr_stat   = 0x3F;
     b->addr_wdays  = 0x41;
     b->addr_hour   = 0x42;
     b->addr_min    = 0x43;
     b->addr_sec    = 0x44;
     b->shift_wdays = 1;
}

void soyo_sy5ssm(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x34);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->addr_mon    = 0x32;
     b->addr_date   = 0x33;
     b->shift_mon   = 4;
}

void msi_ms_6524(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
}

void qdi_brlx1(struct biosinfo *b)
{
     ga_6bxc_qdi_common(b);
     b->need_reboot = ON_STAT;
}

void ecs_k7s5a(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x51);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x50;
     b->shift_stat  = 7;
     b->shift_date  = 3;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;

     b->rtc_date_0_is_c0 = ON;
}

void ecs_k7som_p(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x51);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x4F;
     b->shift_stat  = 7;
     b->shift_date  = 3;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;

     b->rtc_date_0_is_c0 = ON;
}

void ecs_k7s5al(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x52);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x50;
     b->shift_stat  = 7;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;

     b->rtc_date_0_is_c0 = ON;
}

void ecs_k7s5a_1(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x52);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x51;
     b->shift_stat  = 6;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;

     b->rtc_date_0_is_c0 = ON;
}

void ecs_k7s5au(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x52);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x51;
     b->shift_stat  = 7;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;

     b->rtc_date_0_is_c0 = ON;
}

void gig_ga7vkml(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x57);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x56;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
     b->shift_stat  = 3;
}

void asus_k7m(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x57);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x56;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
     b->shift_stat  = 4;
}

void gig_ga_7zxr30(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x57);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x56;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
     b->shift_stat  = 5;
}

void fic_sd_11(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x56);
     b->need_reboot = ON_STAT;
     b->addr_stat   = 0x55;
     b->shift_stat  = 4;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void gig_ga_5ax(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     BCD(b);
     b->addr_chk_h2 = 0x2B; /* Checksum for the status byte */
     b->addr_chk_l2 = 0x2C; /* Checksum for the status byte */
     b->need_reboot = ALWAYS;
     b->rtc_time    = ON;
     b->addr_stat   = 0x2A;
     b->addr_date   = 0x44;
     b->addr_hour   = 0x45;
     b->addr_min    = 0x46;
     b->shift_stat  = 7;
}

void mp_avt_121(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x4F);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x4E;
     b->shift_stat  = 2;
}

void mp_avt_122(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x53);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x53;
     b->shift_stat  = 2;
     b->shift_date  = 3;
}

void msi_6535_20b(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x55);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x53;
     b->shift_stat  = 7;
     b->shift_date  = 3;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
}

void asrock_g_pro(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x55);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x55;
     b->shift_date  = 1;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
}

void asus_p4t533(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4C;
     b->addr_date   = 0x4C;
     b->shift_stat  = 5;
     b->shift_date  = 2;
     b->rtc_time    = ON;
     b->date_hack   = 2;
}

void asus_a7v8x(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);

     b->need_reboot = OFF; /* ??? */

     b->addr_stat   = 0x4D;
     b->shift_stat  = 5;

     b->addr_date   = 0x4E;
     b->date_hack   = 2;

     b->rtc_time    = ON;
}

void asus_p4t533_c(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4C;
     b->shift_stat  = 5;
     b->rtc_time    = ON;
     b->addr_date   = 0x4D;
     b->date_hack   = 2;
}

void asus_p4s8x(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4C;
     b->shift_stat  = 3;
     b->rtc_time    = ON;
     b->addr_date   = 0x4D;
     b->date_hack   = 4;
}

void asus_p4pe(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     TIMEREGS(b, 0x67);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4E;
     b->shift_stat  = 3;
     b->addr_date   = 0x4F;
     b->date_hack   = 4;
     BCD(b);
     b->nr_date     = 5;
/*   DATE is not (!) in BCD. This does not matter, since a special 
     (date_hack) handling required anyway. just nr_date must be correct! */
}

void ecs_k7som(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x52);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x52;
     b->shift_date  = 1;
}

void via_p4ma(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     TIMEREGS(b, 0x48);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x47;
     b->shift_stat  = 3;
}

void dfi_ak70(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     TIMEREGS(b, 0x48);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x46;
     b->shift_stat  = 2;
}

void via_epia_m(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     TIMEREGS(b, 0x4C);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x4B;
     b->shift_stat  = 3;
}

void msi_694tpro(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x58);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x57;
     b->shift_stat  = 5;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void gig_ga_7vtx(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x55);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x55;
     b->shift_stat  = 2;
     b->shift_date  = 3;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void gig_ga_7ixe4(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x55);
     b->need_reboot = ON_STAT;
     b->addr_stat   = 0x54;
     b->shift_stat  = 6;
     b->rtc_date    = 0x6F;
     b->reset_date  = ON;
}

void acer_s58m(struct biosinfo *b)
{
     /* This board doesn't need a checksum */
     BCD(b);
     b->need_reboot = OFF;
     b->addr_stat   = 0x41;
     b->addr_hour   = 0x54;
     b->addr_min    = 0x55;
     b->addr_sec    = 0x56;
     b->rtc_date    = 0x5C;
     b->shift_stat  = 4;
}

void msi_k7pro_k7pro18_ecs_common(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x58);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x57;
}

void msi_k7_pro(struct biosinfo *b)
{
     msi_k7pro_k7pro18_ecs_common(b);
}

void ecs_k7vmm(struct biosinfo *b)
{
     msi_k7pro_k7pro18_ecs_common(b);
     b->shift_stat  = 3;
}

void msi_ms_6243(struct biosinfo *b)
{
     msi_k7pro_k7pro18_ecs_common(b);
     b->shift_stat  = 5;
}

void msi_k7pro18(struct biosinfo *b)
{
     msi_k7pro_k7pro18_ecs_common(b);
     b->shift_stat  = 7;
}

void msi_6535_oldbios(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x55);
     b->addr_stat   = 0x54;
     b->shift_date  = 2;
     b->shift_stat  = 7;
     b->rtc_date    = 0x70;
     b->nr_rtc_date = 8;
     b->reset_date  = ON;
     b->rtc_date_0_is_c0 = ON;
}

void msi_6535(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x56);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x55;
     b->shift_stat  = 4;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
     b->nr_rtc_date = 8;
     b->rtc_date_0_is_c0 = 1;
}

void ecs_k7sem(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->addr_mon    = 0x32;
     b->shift_mon   = 4;
}

void gig_sis_645(struct biosinfo *b)
{
     /* ACPI must be disabled in Kernel, otherwise PC would'nt wake up! */
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->addr_mon    = 0x32;
     b->shift_mon   = 4;

     b->rtc_time    = ON;
}

void sis_630s(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->addr_mon    = 0x32;
     b->shift_mon   = 4;

     b->rtc_time    = ON;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
     b->rtc_date_0_is_c0 = ON;
}

void shuttle_fs40(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->addr_mon    = 0x32;
     b->shift_mon   = 4;
     b->rtc_time    = ON;
     b->rtc_date    = 0x70;
     b->rtc_mon     = 0x71;
     b->reset_date  = ON;
     b->reset_mon   = ON;
     b->rtc_date_0_is_c0 = ON;
     b->rtc_mon_0_is_c0  = ON;
}

void ecs_k7s6a(struct biosinfo *b)
{
     CHECKSUM(b, 0x6E);
     TIMEREGS(b, 0x33);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x32;
     b->rtc_time    = ON;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
     b->rtc_date_0_is_c0 = ON;
}

void chaintech_8363(struct biosinfo *b)
{
     CHECKSUM(b, 0x6D);
     b->need_reboot = ALWAYS;
     b->addr_min    = 0x40;
     b->addr_hour   = 0x41;
     b->addr_date   = 0x51;
     /* addr_stat is missing -- ??? */
}

void asus_a7v333(struct biosinfo *b)
{
     CHECKSUM(b,0x6B);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4B;
     b->shift_stat  = 1;
     b->addr_date   = 0x4B;
     b->shift_date  = 2;
     b->rtc_time    = ON;
}

void asus_a7s333(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF; /* ??? */
     b->addr_stat   = 0x4B;
     b->shift_stat  = 6;
     b->addr_date   = 0x4C;
     b->rtc_time    = ON;
}

void asus_a4s333(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF;
     b->addr_stat   = 0x4B;
     b->shift_stat  = 6;
     b->addr_date   = 0x4C;
     b->date_hack   = 1;
     b->rtc_time    = ON;
}

void asus_p4b266(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);

     b->need_reboot = OFF; /* ??? */

     b->addr_stat   = 0x4B;
     b->shift_stat  = 5;

     b->addr_date   = 0x4C;
     b->date_hack   = 2;

     b->rtc_time    = ON;
}

void asus_p4b266_m(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->need_reboot = OFF;
     b->addr_stat   = 0x4A;
     b->shift_stat  = 4;
     b->addr_date   = 0x4B;
     b->date_hack   = 3;
     b->rtc_time    = ON;
}

void pcchips_810lmr(struct biosinfo *b)
{
     CHECKSUM(b, 0x30);
     TIMEREGS(b, 0x5B);
     b->need_reboot = ALWAYS;
     b->addr_stat   = 0x5A;
     b->shift_stat  = 3;
     b->rtc_date    = 0x70;
     b->reset_date  = ON;
}

void dell_optiplex_dxa(struct biosinfo *b)
{
     CHECKSUM(b, 0x5F);  /* this one calculates the checksums in a different way! */
                         /* define DELL_CHK in beginning of nvram-wakeup.c        */
     
     b->addr_stat  = 0x4B;
     b->shift_stat = 6;  /* this bit switches between "off" and "everyday"        */

     BCD(b);

     b->addr_hour  = 0x4C;
     b->addr_min   = 0x4D;
}

void asus_p4bgl(struct biosinfo *b)
{
     CHECKSUM(b, 0x6B);
     b->addr_stat  = 0x4C;

     b->addr_date  = 0x4C;  /* the date is not in BCD. define ASUS_P4BGL in beginning of nvram-wakeup.c */
     b->shift_date = 1;

     BCD(b);
     b->nr_date    = 5;

     b->addr_hour  = 0x68;
     b->addr_min   = 0x69;
     b->addr_sec   = 0x6A;
}


/*
 * Definitions of the mainboards below are in the format
 *
 *             { infowriter,        iwname,             boardvendor, boardtype, boardversion, biosvendor, biosversion, biosrelease }
 *
 * Where NULL means "Don't care" and "" is a real empty field.
 */
static struct mainboard boards[] = {
/* MB =  1 */  { msi_ms_6351,      "msi_ms_6351",      "MSI",             "MS-6351", NULL, NULL, NULL, NULL},
               { msi_ms_6351,      "msi_ms_6351",      "Micro-Star Inc.", "MS-6351", "1.0", "American Megatrends Inc.", "62710", "01/14/02" },
               { msi_ms_6351,      "msi_ms_6351",      "Micro-Star Inc.", "MS-6351", "1.0", "American Megatrends Inc.", "62710", "04/03/02" },

/* MB =  2 */  { bio_m6vcg,        "bio_m6vcg",        "Abit <http://www.abit.com.tw/>", "KT7", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Abit <http://www.abit.com.tw/>", "KA7-100", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Abit <http://www.abit.com.tw/>", "VP6", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "<http://www.abit.com.tw>", "8363-686A(KT7[A][-RAID],KT7E)", "v1.0 ~ v1.2", NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Acer", "M19A", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "AOPEN", "AK33", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        /* Aopen AK73 */ "", "07", "9189310401", NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Biostar", "M6VBE", NULL, NULL, NULL, NULL },
               /* 
                *  Unfortunately BIOSTAR M6VCG has no valid DMI entries
                *  (at least bios rev. 07/31/2000). Thus,
                *
                *     --iwname bio_m6vcg
                *
                *  option is your friend.
                */
               { bio_m6vcg,        "bio_m6vcg",        "Biostar", "M6VCG", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        /* Chaintech 6AIV2T */ "", "601-686A", "", "Award Software International, Inc.", "6.00 PG", "09/26/2001" },
               { bio_m6vcg,        "bio_m6vcg",        "Elitegroup", "K7VZA", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Elitegroup", "P6VAP", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Epox", "8KTA", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Shuttle", "FV24", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Shuttle", "MV25N", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "Tyan", "S1834", NULL, NULL, NULL, NULL },
               { bio_m6vcg,        "bio_m6vcg",        "", "MS-6318", "", "Award Software International, Inc.", "6.00 PG", "06/13/01" },
               { bio_m6vcg,        "bio_m6vcg",        /* NMC/ENMIC 7VAX */ "", "8371-686A", "", "Award Software International, Inc.", "6.00 PG", "02/21/2001" },
               { bio_m6vcg,        "bio_m6vcg",        "Legend QDI", "Advance-10T", "", "Award Software International, Inc.", "6.00 PG", "08/23/2001" },
               { bio_m6vcg,        "bio_m6vcg",        "<http://www.abit.com.tw>", "8363-686A(KT7,KT7A,KT7-RAID,KT7A-RAID)", "", "Award Software International, Inc.", "6.00 PG", "07/11/2002" },

/* MB =  3 */  { gig_ga_6bxc,      "gig_ga_6bxc",      "Gigabyte", "GA-6BXC", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "Gigabyte", "GA-6BXD", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "Gigabyte", "GA-6BA", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "Gigabyte", "GA-8GE667 Pro", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      /* Gigabyte GA-BX2000 */ "Corporation Name", "i440BX-8671", "1.0", NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "MSI", "MS-6117", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "MSI", "MS-6156", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "Tekram", "P6B40-A4X", NULL, NULL, NULL, NULL },

               { gig_ga_6bxc,      "gig_ga_6bxc",      "MediaPortal", "QDI Winnex9", NULL, NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      "Tekram Technology Co., Ltd.", "P6B40-A4X-i440BX", "Rev. 1.0", NULL, NULL, NULL },
               { gig_ga_6bxc,      "gig_ga_6bxc",      /* from Shuttle barebone SB51G */ "Shuttle Inc", "FB50", "", "Phoenix Technologies, LTD", "6.00 PG", "11/14/2002" }, 
               { gig_ga_6bxc,      "gig_ga_6bxc",      /* Chaintech 6LTM2 */ "", "i440LX-8671", "", "Award Software International, Inc.", "4.51 PG", "07/20/99" },

/* MB =  4 */  { epx_8kta3,        "epx_8kta3",        "EPox", "8KTA3", NULL, NULL, NULL, NULL },
     
/* MB =  5 */  { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "TUSL2", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "TUSL2-C", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "CUSL2", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "CUSL2-C", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "P3B-F", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "<P3B-F>", NULL, NULL, NULL, NULL },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "<CUBX>", "REV 1.xx", "Award Software, Inc.", "ASUS CUBX ACPI BIOS Revision 1008 Beta 004", "10/15/2001" },
               { asus_tusl2,       "asus_tusl2",       "ASUSTeK Computer INC.", "<CUBX-L>", "REV 1.xx", "Award Software, Inc.", "ASUS CUBX-L/CUBX-E ACPI BIOS Revision 1007.A", "07/26/2000" },

/* MB =  6 */  { ecs_p6bat,        "ecs_p6bat",        "AOPEN", "AX59PRO", NULL, NULL, NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        "DFI", "K6BV3+", "", "Award Software International, Inc.", "4.51 PG", "01/15/01" },
               { ecs_p6bat,        "ecs_p6bat",        "DFI", "K6BV3+/66", NULL, NULL, NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        /* ECS Elitegroup P6BAP-Me */ "", "693-596-W977EF", "", "Award Software International, Inc.", "4.51 PG", "06/22/00" },
               { ecs_p6bat,        "ecs_p6bat",        "Elitegroup", "P6BAT-A+", NULL, NULL, NULL, NULL },
               /* 
                *  Unfortunately EPOX MVP3C-M has no valid DMI entries
                *  (at least bios rev. 1.01). Thus,
                *
                *     --iwname ecs_p6bat
                *
                *  option is your friend.
                */
               { ecs_p6bat,        "ecs_p6bat",        "EPOX", "MVP3C-M (EP-58)", "Rev 1.0", "Award", NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        "Shuttle", "HOT-661V", NULL, NULL, NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        "Shuttle", "AV61", NULL, NULL, NULL, NULL },
/* MB = 32 */  { ecs_p6bat,        "ecs_p6bat",        "NMC", "5VMXe", NULL, NULL, NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        "Apollo", "MVP3", NULL, NULL, NULL, NULL },
               { ecs_p6bat,        "ecs_p6bat",        /* ECS P6BAT-A+ */ "", "693-596-W977EF", "", "Award Software International, Inc.", "4.51 PG", "06/22/00" },
     

               { shuttle_mvp3,     "shuttle_mvp3",     "Shuttle Inc.", "VIA APOLLO MVP3 (HOT-591P)", "2A5LEH2A", NULL, NULL, NULL },

/* MB =  7 */  { msi_ms_6120,      "msi_ms_6120",      "MSI", "MS-6120", NULL, NULL, NULL, NULL },

/* MB =  8 */  { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "CUV4X-E", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "A7M266", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "A7V-133", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "<A7V133>", "REV 1.xx", "Award Software, Inc.", "ASUS A7V-133 ACPI BIOS Revision 1009", "04/23/2002" },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "<A7V133>", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "A7V266-E", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "<K7V>", "REV 1.xx", "Award Software, Inc.", "ASUS K7V 1007", "07/26/2000" },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "TUV4X", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "Gigabyte", "GA-7DXR", NULL, NULL, NULL, NULL },

               /*
                *  the lower two lines of the next few are for the same board with different BIOS revisions.
                *  If you have the missing information on the BIOS entries, please submit
                */
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "<A7V>", "REV 1.xx", "Award Software, Inc.", "ASUS A7V ACPI BIOS Revision 1008", "07/17/2001" },
               { bio_m6vcg,        "bio_m6vcg",        "ASUSTeK Computer INC.", "A7V", NULL, NULL, NULL, NULL },
               { asus_cuv4xe,      "asus_cuv4xe",      "ASUSTeK Computer INC.", "A7V", NULL, NULL, NULL, NULL },

/* MB =  9 */  { asus_a7s_vm,      "asus_a7s_vm",      "ASUSTeK Computer INC.", "A7S-VM", NULL, NULL, NULL, NULL },
               { asus_a7s_vm,      "asus_a7s_vm",      "ASUSTeK Computer INC.", "A7A266-E", NULL, NULL, NULL, NULL },

/* MB = 10 */  { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P2L97-S", NULL, NULL, NULL, NULL },
               { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P2B", NULL, NULL, NULL, NULL },
               { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P2B-B", NULL, NULL, NULL, NULL },
               { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P2B-F", NULL, NULL, NULL, NULL },
               { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P5A", NULL, NULL, NULL, NULL },
               { asus_p2l97s,      "asus_p2l97s",      "ASUSTeK Computer INC.", "P5A-B", "REV 1.XX", "Award Software, Inc.", "ASUS P5A-B ACPI BIOS Revision 1011 Beta 004", "12/14/00" },
     
               { asus_tx97_xe,     "asus_tx97_xe",     "ASUSTeK Computer INC.", "TX97-XE", NULL, NULL, NULL, NULL },
     
/* MB = 11 */  { pb_bora_pro,      "pb_bora_pro",      "Packard Bell", "Bora Pro", NULL, NULL, NULL, NULL },
     
/* MB = 12 */  { fic_va_503p,      "fic_va_503p",      "FIC", "VA 503+", NULL, NULL, NULL, NULL },

/* MB = 13 */  { msi_k7_pro,       "msi_k7_pro",       "MSI", "K7 Pro", NULL, NULL, NULL, NULL },
     
/*  These boards don't support wake-on-timer at all */
/*
 *             { sci_pro_c5,       "sci_pro_c5",       "Sciencic", "Pro C5", NULL, NULL, NULL, NULL },
 *             { sci_pro_c5,       "sci_pro_c5",       "Sciencic", "Pro C6", NULL, NULL, NULL, NULL }, 
 */
     
/* MB = 15 */  { msi_ms_6368,      "msi_ms_6368",      "MSI", "MS-6368", NULL, NULL, NULL, NULL },
               { msi_ms_6368,      "msi_ms_6368",      "MSI", "MS-6330", "v5.0", NULL, NULL, NULL },
               { msi_ms_6368,      "msi_ms_6368",      "", "MS-6318", "", "Award Software International, Inc.", "6.00 PG", "09/28/00" },
               { msi_ms_6368,      "msi_ms_6368",      "MICRO-STAR INTERNATIONAL CO., LTD", "MS-6378(VT8361)", "", NULL, NULL, NULL },
               { msi_ms_6368,      "msi_ms_6368",      "Abit", "694X-686B", "VH6T 1.1", "Award Software International, Inc.", "6.00 PG", "03/15/2002" },
               { msi_ms_6368,      "msi_ms_6368",      "Legend.QDI,Inc", "KINETIZ 7T/7B/7E", "V2.X", "Award Software International, Inc.", "6.00 PG", "03/07/2003" },
               { msi_ms_6368,      "msi_ms_6368",      "Legend.QDI,Inc", "8363-686A", "KINETIZ 7T", "Award Software International, Inc.", "6.00 PG", "03/07/2003" },

               { abit_kt7e,        "abit_kt7e",        "<http://www.abit.com.tw>", "8363-686A(KT7,KT7A,KT7A-RAID,KT7E)", "", NULL, NULL, NULL },

/* MB = 16 */  { gig_ga_7zx,       "gig_ga_7zx",       "Gigabyte", "GA-7ZX", NULL, NULL, NULL, NULL },
     
/* MB = 17 */  { asus_sp98_x,      "asus_sp98_x",      "ASUSTeK Computer INC.", "SP98AGP-X", NULL, NULL, NULL, NULL },
     
/* MB = 18 */  { soyo_sy5ssm,      "soyo_sy5ssm",      "Soyo", "SY-5SSM", NULL, NULL, NULL, NULL },
               { soyo_sy5ssm,      "soyo_sy5ssm",      "Gigabyte", "GA-586STX2", NULL, NULL, NULL, NULL },

/* MB = 19 */  { msi_k7pro18,      "msi_k7pro18",      "MSI", "K7 Pro", NULL, NULL, NULL, NULL },
     
/* MB = 20 */  { qdi_brlx1,        "qdi_brlx1",        "QDI", "Legend", NULL, NULL, NULL, NULL },
               { qdi_brlx1,        "qdi_brlx1",        "QDI", "BrillianX I", NULL, NULL, NULL, NULL },
               { qdi_brlx1,        "qdi_brlx1",        "QDI", "P6I440BX/B1", NULL, NULL, NULL, NULL },

/* MB = 21 */  { ecs_k7s5a,        "ecs_k7s5a",        "Elitegroup", "K7S5A", NULL, NULL, NULL, NULL },
               { ecs_k7s5a,        "ecs_k7s5a",        "ECS", "K7S5A", "1.0", "American Megatrends Inc.", "07.00T", "04/02/01" },
               { ecs_k7s5a,        "ecs_k7s5a",        "ECS", "K7S5A", NULL, NULL, NULL, NULL },

/* MB = 24 */  { ecs_k7s5al,       "ecs_k7s5al",       "Elitegroup", "K7S5AL", NULL, NULL, NULL, NULL },
     
/* MB = 43 */  { ecs_k7s5au,       "ecs_k7s5au",       "Elitegroup", "K7S5AU", NULL, NULL, NULL, NULL },
     
               { ecs_k7s5a_1,      "ecs_k7s5a_1",      "ECS", "K7S5A", "1.0", "American Megatrends Inc.", "07.00T", "04/02/01" },
     
/* MB = 22 */  { gig_ga7vkml,      "gig_ga7vkml",      "Gigabyte", "GA-7VKML", NULL, NULL, NULL, NULL },

/* MB = 23 */  { fic_sd_11,        "fic_sd_11",        "FIC", "SD 11", NULL, NULL, NULL, NULL },
               { fic_sd_11,        "fic_sd_11",        "FIC", "SD11", NULL, NULL, NULL, NULL },

/* MB = 25 */  { gig_ga_7zxe,      "gig_ga_7zxe",      "Gigabyte", "GA-7ZXE", NULL, NULL, NULL, NULL },
               { gig_ga_7zxe,      "gig_ga_7zxe",      "Gigabyte Technology Co., Ltd.", "7IXEH", "1.x", "American Megatrends Inc.", "62710", "10/08/2001" },

/* MB = 26 */  { gig_ga_5ax,       "gig_ga_5ax",       "Gigabyte", "GA-5AX", NULL, NULL, NULL, NULL },

/* MB = 27 */  { mp_avt_121,       "mp_avt_121",       "MediaPortal", "AVT", "i810 bios version 1.21", NULL, NULL, NULL },

/* MB = 28 */  { mp_avt_122,       "mp_avt_122",       "MediaPortal", "AVT", "i810 bios version 1.22", NULL, NULL, NULL },

/* MB = 30 */  { asus_p4t533_c,    "asus_p4t533_c",    "ASUSTeK Computer INC.", "P4T533-C", NULL, NULL, NULL, NULL },

/* MB = 46 */  { asus_p4pe,        "asus_p4pe",        "ASUSTeK Computer INC.", "P4PE", NULL, NULL, NULL, NULL },

/* MB = 31 */  { gig_ga_7vtxh,     "gig_ga_7vtxh",     "Gigabyte", "GA-7VTXH", NULL, NULL, NULL, NULL },
               { gig_ga_7vtxh,     "gig_ga_7vtxh",     "Gigabyte Technology Co., Ltd.", "7VTXH", "1.0", "American Megatrends Inc.", "062710", "12/19/2001" },
               { gig_ga_7vtxh,     "gig_ga_7vtxh",     "Gigabyte", "GA-7VTXE", NULL, NULL, NULL, NULL },

/* MB = 33 */  { msi_ms_6380,      "msi_ms_6380",      "MSI", "MS-6380", NULL, NULL, NULL, NULL },
     
/* MB = 34 */  { ecs_k7som,        "ecs_k7som",        "Elitegroup", "K7SOM", NULL, NULL, NULL, NULL },

/* MB = 35 */  { via_p4ma,         "via_p4ma",         "VIA", "P4MA", NULL, NULL, NULL, NULL },
               { via_p4ma,         "via_p4ma",         "", "P4MA", "", "Award Software International, Inc.", "6.00 PG", "05/23/2002" },

/* MB = 36 */  { msi_694tpro,      "msi_694tpro",      "MSI", "MS-6309", NULL, NULL, NULL, NULL },

/* MB = 37 */  { gig_ga_7vtx,      "gig_ga_7vtx",      "Gigabyte", "GA-7VTXH", NULL, NULL, NULL, NULL },
               { gig_ga_7vtx,      "gig_ga_7vtx",      "Gigabyte Technology Co., Ltd.", "7VTXE", "1.0", "American Megatrends Inc.", "07.00T", "06/28/2002" },

/* MB = 38 */  { acer_s58m,        "acer_s58m",        "Acer", "S58M", NULL, NULL, NULL, NULL },
     
/* MB = 39 */  { ecs_k7vmm,        "ecs_k7vmm",        "Elitegroup", "K7VMM", NULL, NULL, NULL, NULL },

               /* 
                * Unfortunately MSI changed the BIOS without increasing the BIOS release number
                * or revision date, so the right thing to do can't be detected automatically.
                *
                * The first line is for BIOS 1.2, the second one is for 1.0 or 1.1
                *
                * if it doesn't work for you out-of-the-box, either update the BIOS 
                * or use msi_6535_oldbios (--iwname option is your friend). 
                */
               { msi_6535,         "msi_6535",         "MSI", "MS-6535", "1.00", "American Megatrends Inc.", "07.00T", "12/14/01" },
               { msi_6535_oldbios, "msi_6535_oldbios", "MSI", "MS-6535", "1.00", "American Megatrends Inc.", "07.00T", "12/14/01" },

               { msi_6535_20b,     "msi_6535_20b",     "MSI", "MS-6535", "2.0B", "American Megatrends Inc.", "Version 07.00T", "04/06/02" },

               { sis_630s,         "sis_630s",         "", "SiS-630S", "", "Award Software International, Inc.", "6.00 PG", "04/08/2002" },

               { chaintech_8363,   "chaintech_8363",   /* Chaintech CT-7AJA-2 */ "", "8363-686A", "", "Award Software International, Inc.", "6.00 PG", "04/23/2001" },

/* MB = 40 */  { asus_a7v333,      "asus_a7v333",      "ASUSTeK Computer INC.", "A7V333", NULL, NULL, NULL, NULL },
     
               { asus_a7s333,      "asus_a7s333",      "ASUSTeK Computer INC.", "A7S333", "REV 1.xx", "Award Software, Inc.", "ASUS A7S333 ACPI BIOS Revision 1005 Beta 003", "10/30/2002" },

/* MB = 41 */  { msi_ms_6380e,     "msi_ms_6380e",     "MSI", "MS-6380E", NULL, NULL, NULL, NULL },

/* MB = 42 */  { asus_p4t533,      "asus_p4t533",      "ASUSTeK Computer INC.", "P4T533/R-UAY", NULL, NULL, NULL, NULL },

/* MB = 44 */  { via_epia_m,       "via_epia_m",       "VIA", "Epia M6000", NULL, NULL, NULL, NULL },
               { via_epia_m,       "via_epia_m",       "VIA", "Epia M9000", NULL, NULL, NULL, NULL },
               { via_epia_m,       "via_epia_m",       /*         M10000 */ "", "EPIA-M", "", "Award Software International, Inc.", "6.00 PG", "11/19/2002" },
               { via_epia_m,       "via_epia_m",       /*         M10000 */ "", "EPIA-M", "", "Award Software International, Inc.", "6.00 PG", "11/26/2002" },
               { via_epia_m,       "via_epia_m",       /* M9000 + M10000 */ "", "EPIA-M", "", "Award Software International, Inc.", "6.00 PG", "03/17/2003" },
     
/* MB = 45 */  { gig_ga_7zxr30,    "gig_ga_7zxr30",    "Gigabyte", "GA-7ZXR30", NULL, NULL, NULL, NULL },

/* MB = 47 */  { gig_ga_7ixe4,     "gig_ga_7ixe4",     "Gigabyte", "GA-7IXE4", NULL, NULL, NULL, NULL },
               { gig_ga_7ixe4,     "gig_ga_7ixe4",     "Gigabyte Technology Co., Ltd.", "7IXE", "1.1", "American Megatrends Inc.", "62710", "06/01/2000" },

/* MB = 48 */  { ecs_k7sem,        "ecs_k7sem",        "Elitegroup", "K7SEM", NULL, NULL, NULL, NULL },
               { shuttle_fs40,     "shuttle_fs40",     /* Shuttle FS40 */ "", "", "", "Phoenix Technologies, LTD", "6.00 PG", "04/30/2002" },
               { shuttle_fs40,     "shuttle_fs40",     /* Shuttle FS40 */ "", "FS40", "", "Phoenix Technologies, LTD", "6.00 PG", "01/21/2003" },

               { ecs_k7s6a,        "ecs_k7s6a",        /* ECS K7S6A */ "", "SiS-745", "", "Award Software International, Inc.", "6.00 PG", "02/19/2002" },
               
               { gig_sis_645,      "gig_sis_645",      "Gigabyte Technology Co., Ltd.", "SiS-645", "1.x", "Award Software International, Inc.", "6.00 PG", "02/06/2002" },

               { msi_ms_6243,      "msi_ms_6243",      /* MSI-Barebone Hermes 845GL   (MS-6243) */ "INTEL", "BROKDALE-G", "1.0", "American Megatrends Inc.", "V1.1", "06/10/02" },
               { msi_ms_6243,      "msi_ms_6243",      /* MSI-Barebone Hermes 845GL-P (MS-6557) */ "INTEL", "BROKDALE-G", NULL, NULL, NULL, NULL },

               { asrock_k7vt2,     "asrock_k7vt2",     /* ASRock K7VT2 */ "", "K7VT2", "2.00", "American Megatrends Inc.", "P1.20", "12/10/2002" },

               { asrock_k7vm2,     "asrock_k7vm2",     "ASRock", "K7VM2", "2.00", "American Megatrends Inc.", "P1.30", "12/13/2002" },
               { asrock_k7vm2,     "asrock_k7vm2",     "", "K7VM2", "2.00", "American Megatrends Inc.", "P1.30", "12/13/2002" },

               { asus_p4s8x,       "asus_p4s8x",       "ASUSTeK Computer INC.", "P4S8X", "REV 1.xx", "Award Software, Inc.", "ASUS P4S8X ACPI BIOS Revision 1005 Beta 009", "02/21/2003" },

               { msi_ms_6524,      "msi_ms_6524",      "MICRO-STAR INTERNATIONAL CO., LTD", "MS-6524", "", "Award Software International, Inc.", "6.00 PG", "04/26/2002" },

               { pcchips_810lmr,   "pcchips_810lmr",   /* pcchips 810LMR version 7.1a */ "", "", "", "American Megatrends Inc.", "062710", "07/15/97" },

               { msi_ms_6712,      "msi_ms_6712",      "MSI", "MS-6712", "1.0", "American Megatrends Inc.", "Version 07.00T", "04/02/01" },
               
               { asus_k7m,         "asus_k7m",         "ASUSteK Computer INC.", "K7M", "1.XX", "American Megatrends Inc.", "627.10", "02/29/2000" },
 
               { msi_ms_6160,      "msi_ms_6160",      /* Fujitsu-Siemens-PC */ "MICRO-STAR INTERNATIONAL CO., LTD", "MS-6160 (i440LX)", "1.X", "Award Software International, Inc.", "4.51 PG", "10/20/98" },

               { asus_a7v8x,       "asus_a7v8x",       "ASUSTeK Computer INC.", "A7V8X", "REV 1.xx", "Award Software, Inc.", "ASUS A7V8X ACPI BIOS Revision 1007", "10/04/2002" },

               { abit_bp6,         "abit_bp6",         "<http://www.abit.com.tw>", "i440BX-W83977 (BP6)", "", "Award Software International, Inc.", "4.51 PG", "04/20/00" },
               
               { aopen_ax4pe_max,  "aopen_ax4pe_max",  "AOpen", "AX4PE Max", "", "Phoenix/Award Technologies, LTD", "6.00 PG", "12/23/2002" },             
               
               { msi_ms_6570,      "msi_ms_6570",      "MICRO-STAR INTERNATIONAL CO., LTD", "MS-6570", "", "Phoenix Technologies, LTD", "6.00 PG", "04/18/2003" },

               { asus_a7n8x,       "asus_a7n8x",       "ASUSTeK Computer INC.", "A7N8X", "REV 1.xx", "Phoenix Technologies, LTD", "ASUS A7N8X Deluxe ACPI BIOS Rev 1002.A", "02/25/2003" },

               { ecs_k7som_p,      "ecs_k7som_p",      "ECS", "K7SOM+", "1.0", "American Megatrends Inc.", "07.00T", "04/02/01" },

               { asus_a4s333,      "asus_a4s333",      "ASUSTeK Computer INC.", "P4S333VF", "REV 1.xx", "Award Software, Inc.", "ASUS P4S333-VF ACPI BIOS Revision 1005", "03/05/2002" },

               { dell_optiplex_dxa,"dell_optiplex_dxa",/* Dell Optiplex DXa */ NULL, NULL, NULL, "Dell Computer Corporation", "A10", "06/23/00" },

               { asus_p4bgl,       "asus_p4bgl",       "ASUSTeK Computer INC.", "P4BGL-MX", "REV 1.xx", "Award Software, Inc.", "ASUS P4BGL-MX ACPI BIOS Revision 1006", "02/27/2003" },

               { asus_p4b266,      "asus_p4b266",      "ASUSTeK Computer INC.", "P4B266",   "REV 2.xx", "Award Software, Inc.", "ASUS P4B266 ACPI BIOS Revision 1010", "08/06/2002" },
               { asus_p4b266_m,    "asus_p4b266_m",    "ASUSTeK Computer INC.", "P4B266-M", "REV 1.xx", "Award Software, Inc.", "ASUS P4B266-M ACPI BIOS Revision 1008 Beta 001", "04/03/2003" },
               
               { msi_ms_6593,      "msi_ms_6593",      "MSI", "MS-6593", "1.0", "American Megatrends Inc.", "07.00T", "04/02/01" },
               { msi_ms_6590,      "msi_ms_6590",      "MSI", "MS-6590", "1.0", "American Megatrends Inc.", "Version 07.00T", "04/02/01" },

               { chaintech_6aja4,  "chaintech_6aja4",  /* Chaintech CT-6AJA4/G100 */ "", "694X-686A", "", "Award Software International, Inc.", "6.00 PG", "05/17/2001" },

               { dfi_ak70,         "dfi_ak70",         /* DFI AK70 Rev. A+ */ "", "AMD-75X-W977", "", "Award Software International, Inc.", "6.00 PG", "05/25/2000" },

               { asrock_g_pro,     "asrock_g_pro",     "ASRock", "G Pro / GV Pro", "1.00", "American Megatrends Inc.", "P1.40", "04/07/ 2003" },
               
               { asus_p4s533,      "asus_p4s533",      "ASUSTeK Computer INC.", "P4S533MX", "REV 1.xx", "Award Software, Inc.", "ASUS P4S533-MX ACPI BIOS Revision 1004", "01/24/2003" },
               
               { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
};

int set_biosinfo_defaults(struct biosinfo *b) {
     if (b) {
          memset(b, 0, sizeof(struct biosinfo));
          b->nr_stat = 1;
          b->addr_stat = 0xFF;
          NOBCD(b);
          return 1;
     }
     else return 0;
}

int get_bios_info_by_dmi(struct biosinfo *b) {

     int i;

     /* Set defaults... */
     if (!set_biosinfo_defaults(b))
          return 0;

     if (!__dmi_probe())
          return -1;

     /* Check if we know the mainboard... */
     for(i=0; boards[i].infowriter; i++) {
          if (boards[i].vendor      && (!board_vendor()  || strcmp(boards[i].vendor,      board_vendor() )))
               continue;
          if (boards[i].type        && (!board_type()    || strcmp(boards[i].type,        board_type()   )))
               continue;
          if (boards[i].version     && (!board_version() || strcmp(boards[i].version,     board_version())))
               continue;
          if (boards[i].biosvendor  && (!bios_vendor()   || strcmp(boards[i].biosvendor,  bios_vendor()  )))
               continue;
          if (boards[i].biosversion && (!bios_version()  || strcmp(boards[i].biosversion, bios_version() )))
               continue;
          if (boards[i].biosrelease && (!bios_release()  || strcmp(boards[i].biosrelease, bios_release() )))
               continue;

          /* This is our board... */
          boards[i].infowriter(b);
          return 1;
     }
     return 0;
}



int get_bios_info_by_iw(struct biosinfo *b, const char * iw_name) {

     int i;

     /* Set defaults... */
     if (!set_biosinfo_defaults(b))
          return 0;

     /* Check if we know that infowriter... */
     for(i=0; boards[i].infowriter; i++) {
          if ( strcmp( boards[i].iw_name, iw_name ) )
               continue;

          /* This is our board... */
          boards[i].infowriter(b);
          return 1;
     }
     return 0;
}


